/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.dept.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.excel.dept.DepartmentExportExcel;
import com.yiring.app.excel.dept.DepartmentImportExcel;
import com.yiring.app.feign.zy.dept.DepartmentClient;
import com.yiring.app.param.dept.DepartmentAddParam;
import com.yiring.app.param.dept.DepartmentExportParam;
import com.yiring.app.param.dept.DepartmentFindParam;
import com.yiring.app.param.dept.DepartmentModifyParam;
import com.yiring.app.param.zy.dept.ZyDepartmentAddParam;
import com.yiring.app.param.zy.dept.ZyDepartmentFindParam;
import com.yiring.app.param.zy.dept.ZyDepartmentModifyParam;
import com.yiring.app.service.dept.DepartmentService;
import com.yiring.app.util.zy.ZyUtil;
import com.yiring.app.vo.dept.DepartmentInfoVo;
import com.yiring.app.vo.dept.DepartmentVo;
import com.yiring.auth.domain.dept.Department;
import com.yiring.auth.domain.dept.DepartmentRepository;
import com.yiring.auth.domain.post.Post;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.domain.user.UserRepository;
import com.yiring.auth.param.IdsParam;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * 部门管理
 * @author LJ-2204
 * @date 2022/4/19
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class DepartmentServiceImpl implements DepartmentService {

    final Integer FACTORY_ID = 100;

    @Resource
    DepartmentRepository departmentRepository;

    @Resource
    UserRepository userRepository;

    @Resource
    DepartmentClient departmentClient;

    @Override
    public Result<String> addDepartment(DepartmentAddParam departmentAddParam) {
        // 查询用户相关信息
        Optional<User> userOptional = userRepository.findOne(
            Example.of(User.builder().id(departmentAddParam.getLeaderId()).build())
        );
        Department department = DepartmentAddParam.transform(departmentAddParam);
        userOptional.ifPresent(department::setLeader);

        // 查询zy上级部门id
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(departmentAddParam.getId()).build())
        );

        if (departmentOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST);
        }

        Long linkId = departmentOptional.get().getLinkId();

        // 向zy发送新增部门请求
        ZyDepartmentAddParam zyDepartmentAddParam = ZyDepartmentAddParam.transform(department, FACTORY_ID);
        zyDepartmentAddParam.setParentId(linkId);
        JSONObject jsonObject = departmentClient.deptAdd(
            JSONUtil.toJsonStr(zyDepartmentAddParam),
            ZyUtil.clientLogin()
        );
        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            return Result.no(Status.BAD_REQUEST, StrUtil.toString(jsonObject.get("msg")));
        }

        ZyDepartmentFindParam zyDepartmentFindParam = ZyDepartmentFindParam
            .builder()
            .ancestors(Convert.toStr(FACTORY_ID))
            .orgId(Convert.toStr(zyDepartmentAddParam.getOrgId()))
            .deptName(zyDepartmentAddParam.getDeptName())
            .status(zyDepartmentAddParam.getStatus())
            .build();

        Object id = getLinkId(zyDepartmentAddParam, zyDepartmentFindParam);

        department.setLinkId(Convert.toLong(id));
        departmentRepository.save(department);
        return Result.ok();
    }

    @Override
    public Result<PageVo<DepartmentVo>> findDepartmentTree(DepartmentFindParam departmentFindParam) {
        List<Department> departments = departmentRepository.findAll(
            Example.of(Department.builder().enable(departmentFindParam.getEnable()).build())
        );
        List<DepartmentVo> departmentVos = departments
            .stream()
            .map(DepartmentVo::transform)
            .collect(Collectors.toList());
        List<DepartmentVo> departmentVoList = getChildrenList(0L, null, departmentVos);
        PageVo<DepartmentVo> pageVo = PageVo.build(departmentVoList, departmentVos.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<DepartmentInfoVo> findDepartmentInfo(IdParam idParam) {
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(idParam.getId()).build())
        );
        if (departmentOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "部门不存在");
        String pName = null;
        Department department = departmentOptional.get();

        // 渲染上级部门
        if (!ObjectUtil.equals(department.getPid(), 0L)) {
            Optional<Department> department_pid = departmentRepository.findOne(
                Example.of(Department.builder().id(department.getPid()).build())
            );
            if (department_pid.isEmpty()) return Result.no(Status.BAD_REQUEST);
            pName = department_pid.get().getName();
        }
        DepartmentInfoVo departmentInfoVo = DepartmentInfoVo.transform(department);
        departmentInfoVo.setPName(pName);
        return Result.ok(departmentInfoVo);
    }

    @Override
    public void exportDepartment(DepartmentExportParam departmentExportParam, HttpServletResponse response) {
        List<Department> departments = departmentRepository.findAll(
            Example.of(Department.builder().enable(departmentExportParam.getEnable()).build())
        );
        List<DepartmentExportExcel> departmentExportExcels = departments
            .stream()
            .map(DepartmentExportExcel::transform)
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<DepartmentExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(DepartmentExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(departmentExportExcels);

            String fileName = URLEncoder.encode("部门信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出部门信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> deleteDepartment(IdParam idParam) {
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(idParam.getId()).build())
        );

        if (departmentOptional.isEmpty()) return Result.no(Status.BAD_REQUEST);

        JSONObject jsonObject = departmentClient.deptDelete(departmentOptional.get().getLinkId(), ZyUtil.clientLogin());
        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            return Result.no(Status.BAD_REQUEST, StrUtil.toString(jsonObject.get("msg")));
        }
        departmentRepository.deleteById(idParam.getId());
        return Result.ok();
    }

    @Override
    public Result<String> deptBingPost(IdParam idParam, IdsParam idsParam) {
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(idParam.getId()).build())
        );

        if (departmentOptional.isEmpty()) return Result.no(Status.BAD_REQUEST, "部门不存在");

        Department department = departmentOptional.get();

        Long[] ids = Convert.toLongArray(idsParam.getIds().split(","));
        HashSet<Post> posts = new HashSet<>();
        for (Long id : ids) {
            posts.add(Post.builder().id(id).build());
        }

        department.setPosts(posts);

        departmentRepository.save(department);
        return Result.ok();
    }

    @Override
    public Result<String> modifyDept(DepartmentModifyParam departmentModifyParam) {
        Optional<Department> departmentOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(departmentModifyParam.getId()).build())
        );

        if (departmentOptional.isEmpty()) return Result.no(Status.BAD_REQUEST);

        // 查询zy上级部门id
        Optional<Department> pidDeptOptional = departmentRepository.findOne(
            Example.of(Department.builder().id(departmentModifyParam.getPid()).build())
        );

        if (pidDeptOptional.isEmpty()) {
            return Result.no(Status.BAD_REQUEST);
        }

        Department department = departmentOptional.get();
        departmentModifyParam.transform(departmentModifyParam, department);

        Long linkId = pidDeptOptional.get().getLinkId();

        ZyDepartmentModifyParam zyDepartmentModifyParam = ZyDepartmentModifyParam.transform(
            departmentOptional.get(),
            FACTORY_ID
        );
        zyDepartmentModifyParam.setParentId(linkId);
        JSONObject jsonObject = departmentClient.deptModify(
            JSONUtil.toJsonStr(zyDepartmentModifyParam),
            ZyUtil.clientLogin()
        );

        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            return Result.no(Status.BAD_REQUEST, StrUtil.toString(jsonObject.get("msg")));
        }

        departmentRepository.save(department);
        return Result.ok();
    }

    @Override
    public Result<String> importDepartment(MultipartFile file) {
        if (file.isEmpty()) return Result.no(Status.BAD_REQUEST, "请选择文件");
        List<DepartmentImportExcel> departmentImportExcels;
        try (InputStream inputStream = file.getInputStream()) {
            departmentImportExcels =
                SaxExcelReader
                    .of(DepartmentImportExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            return Result.no(Status.BAD_REQUEST, "文件导入异常");
        }

        // 拿到所有部门
        HashMap<String, Department> hashMap = getStringDepartmentHashMap();

        // 对导入信息进行排序
        departmentImportExcels
            .stream()
            .sorted((x, y) -> {
                int xCount = StrUtil.count(x.getName(), "/");
                int yCount = StrUtil.count(y.getName(), "/");
                return Integer.compare(xCount, yCount);
            })
            .forEach(departmentImportExcel -> {
                String name = departmentImportExcel.getName();
                if (ObjectUtil.isNotEmpty(hashMap.get(name))) return;
                int lastIndexOf = name.lastIndexOf("/");
                Department department = DepartmentImportExcel.transform(departmentImportExcel);
                if (lastIndexOf == -1) {
                    throw new RuntimeException("请选择初始厂区");
                } else {
                    String subName = name.substring(0, lastIndexOf);
                    Department hashDept = hashMap.get(subName);

                    ZyDepartmentAddParam zyDepartmentAddParam = ZyDepartmentAddParam.transform(department, FACTORY_ID);
                    zyDepartmentAddParam.setDeptName(name.substring(lastIndexOf + 1));
                    zyDepartmentAddParam.setParentId(hashDept.getLinkId());
                    JSONObject jsonObject = departmentClient.deptAdd(
                        JSONUtil.toJsonStr(zyDepartmentAddParam),
                        ZyUtil.clientLogin()
                    );
                    if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
                        throw new RuntimeException(StrUtil.toString(jsonObject.get("msg")));
                    }

                    ZyDepartmentFindParam zyDepartmentFindParam = ZyDepartmentFindParam
                        .builder()
                        .ancestors(Convert.toStr(FACTORY_ID))
                        .orgId(Convert.toStr(zyDepartmentAddParam.getOrgId()))
                        .deptName(zyDepartmentAddParam.getDeptName())
                        .status(zyDepartmentAddParam.getStatus())
                        .build();

                    Object linkId = getLinkId(zyDepartmentAddParam, zyDepartmentFindParam);
                    department.setName(name.substring(lastIndexOf + 1));
                    department.setPid(hashDept.getId());
                    department.setLinkId(Convert.toLong(linkId));
                    department = departmentRepository.save(department);
                    hashMap.putIfAbsent(name, department);
                }
            });

        return Result.ok();
    }

    public HashMap<String, Department> getStringDepartmentHashMap() {
        List<Department> departments = departmentRepository.findAll();
        HashMap<String, Department> hashMap = new HashMap<>();
        departments.forEach(department -> {
            if (department.getPid() == 0) {
                hashMap.put(department.getName(), department);
            } else {
                ArrayList<String> list = ListUtil.toList();
                List<String> deptNames = getDeptNames(list, department.getPid(), departments);
                ListUtil.reverse(deptNames);
                deptNames.add(department.getName());

                hashMap.put(StrUtil.join("/", deptNames), department);
            }
        });
        return hashMap;
    }

    public List<DepartmentVo> getChildrenList(Long pid, String pName, List<DepartmentVo> departmentVos) {
        return departmentVos
            .stream()
            .filter(departmentVo -> pid.equals(departmentVo.getPid()))
            .sorted(Comparator.comparing(DepartmentVo::getCreateTime).reversed())
            .peek(departmentVo -> {
                departmentVo.setPName(pName);
                departmentVo.setChildList(getChildrenList(departmentVo.getId(), departmentVo.getName(), departmentVos));
            })
            .collect(Collectors.toList());
    }

    public List<String> getDeptNames(ArrayList<String> list, Long pid, List<Department> departmentList) {
        departmentList
            .stream()
            .filter(department -> pid.equals(department.getId()))
            .forEach(department -> {
                list.add(department.getName());
                getDeptNames(list, department.getPid(), departmentList);
            });
        return list;
    }

    private Object getLinkId(ZyDepartmentAddParam zyDepartmentAddParam, ZyDepartmentFindParam zyDepartmentFindParam) {
        // 查询刚刚新增的部门id
        JSONObject result = departmentClient.deptList(zyDepartmentFindParam, ZyUtil.clientLogin());

        if (!ObjectUtil.equals(result.get("code"), 200)) {
            throw new RuntimeException(StrUtil.toString(result.get("msg")));
        }

        JSONArray jsonArray = JSONUtil.parseArray(result.get("data"));
        Optional<Object> objectOptional = jsonArray
            .stream()
            .filter(obj ->
                Convert.toLong(JSONUtil.parseObj(obj).get("parentId")).intValue() ==
                zyDepartmentAddParam.getParentId().intValue()
            )
            .findFirst();
        if (objectOptional.isEmpty()) {
            throw new RuntimeException();
        }

        JSONObject data = JSONUtil.parseObj(objectOptional.get());

        return data.get("deptId");
    }
}
