/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.fence.impl;

import com.alibaba.fastjson.JSONArray;
import com.yiring.app.constant.RedisKey;
import com.yiring.app.domain.location.LocationFence;
import com.yiring.app.domain.location.LocationFenceRepository;
import com.yiring.app.param.location.fence.LocationFenceAddParam;
import com.yiring.app.param.location.fence.LocationFenceConditionParam;
import com.yiring.app.param.location.fence.LocationFenceModifyParam;
import com.yiring.app.service.location.fence.LocationFenceService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.CodeNameVo;
import com.yiring.app.vo.location.fence.LocationFenceSearchVo;
import com.yiring.app.vo.location.fence.LocationFenceVo;
import com.yiring.app.vo.map.MapVo;
import com.yiring.common.core.Redis;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 围栏信息service
 * @author tml
 * @version 1.0
 * @date 2022/4/28 13:47
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class LocationFenceServiceImpl implements LocationFenceService {

    @Resource
    private LocationFenceRepository locationFenceRepository;

    @Resource
    private Redis redis;

    @Override
    public Result<String> addLocationFence(LocationFenceAddParam param) {
        Boolean exists = locationFenceRepository.existsByName(param.getName());
        if (exists) {
            return Result.no(Status.EXPECTATION_FAILED, "围栏名称已被占用");
        }
        LocationFence locationFence = param.transform();
        locationFenceRepository.saveAndFlush(locationFence);
        return Result.ok();
    }

    @Override
    public Result<String> modifyLocationFence(LocationFenceModifyParam param) {
        List<LocationFence> allByName = locationFenceRepository.findAllByName(param.getName());
        if (allByName.size() > 1) {
            log.warn("LocationFenceServiceImpl.modifyLocationFence时查询出两条围栏名称相同的数据,param:[{}]", param);
        }
        LocationFence locationFence = null;
        for (LocationFence item : allByName) {
            if (item.getId().equals(param.getId())) {
                locationFence = item;
            } else {
                return Result.no(Status.EXPECTATION_FAILED, "围栏名称已被占用");
            }
        }
        if (locationFence == null) {
            Optional<LocationFence> optional = locationFenceRepository.findById(param.getId());
            if (optional.isEmpty()) {
                return Result.no(Status.EXPECTATION_FAILED, "要修改的围栏信息不存在");
            }
            locationFence = optional.get();
        }
        JpaUtil.copyNotNullProperties(param, locationFence);
        return Result.ok();
    }

    @Override
    public Result<String> removeLocationFence(IdParam param) {
        Optional<LocationFence> optional = locationFenceRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要删除的围栏信息不存在");
        }
        optional.get().setDeleted(true);
        return Result.ok();
    }

    @Override
    public Result<PageVo<LocationFenceVo>> findList(LocationFenceConditionParam conditionParam, PageParam pageParam) {
        Specification<LocationFence> specification = (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (conditionParam.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get("mapId"), conditionParam.getMapId()));
            }
            if (conditionParam.getFenceType() != null) {
                list.add(criteriaBuilder.equal(root.get("fenceType"), conditionParam.getFenceType()));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
        Pageable pageable = PageParam.toPageable(pageParam);
        Page<LocationFence> fencePage = locationFenceRepository.findAll(specification, pageable);
        List<LocationFenceVo> voList = fencePage.get().map(LocationFenceVo::transform).collect(Collectors.toList());
        PageVo<LocationFenceVo> pageVo = PageVo.build(voList, fencePage.getTotalElements());
        return Result.ok(pageVo);
    }

    @Override
    public Result<LocationFenceVo> findById(IdParam param) {
        Optional<LocationFence> optional = locationFenceRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "查询的围栏信息不存在");
        }
        LocationFence fence = optional.get();
        LocationFenceVo fenceVo = LocationFenceVo.transform(fence);
        return Result.ok(fenceVo);
    }

    @Override
    public Result<PageVo<LocationFenceSearchVo>> findByNameLike(String name) {
        List<LocationFence> list = locationFenceRepository.findLikeName(name);
        List<LocationFenceSearchVo> voList = list
            .stream()
            .map(LocationFenceSearchVo::transform)
            .collect(Collectors.toList());
        PageVo<LocationFenceSearchVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<PageVo<CodeNameVo>> findMapList() {
        Object map = redis.get(RedisKey.ZY_MAP_AREAS);
        String mapString = JSONArray.toJSONString(map);
        List<MapVo.MapVoReuslt> mapVo = JSONArray.parseArray(mapString, MapVo.MapVoReuslt.class);
        List<CodeNameVo> voList = mapVo
            .stream()
            .map(e -> new CodeNameVo(e.getId(), e.getName()))
            .collect(Collectors.toList());
        PageVo<CodeNameVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }
}
