/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.location.tag.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.yiring.app.domain.location.LocationTag;
import com.yiring.app.domain.location.LocationTagRepository;
import com.yiring.app.feign.zy.location.tag.LocationTagTypeClient;
import com.yiring.app.param.location.tag.LocationTagTypeFindParam;
import com.yiring.app.param.location.tag.LocationTagTypeSendParam;
import com.yiring.app.param.zy.location.tag.ZyLocationTagTypeSendParam;
import com.yiring.app.service.location.tag.LocationTagTypeService;
import com.yiring.app.util.zy.ZyUtil;
import com.yiring.app.vo.location.tag.LocationTagTypeVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.domain.BasicEntity;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 标签分配
 *
 * @author LJ-2204
 * @date 2022/4/27
 */

@Transactional(rollbackFor = RuntimeException.class)
@Service
@Slf4j
public class LocationTagTypeServiceImpl implements LocationTagTypeService {

    final Integer FACTORY_ID = 100;

    @Resource
    LocationTagRepository locationTagRepository;

    @Resource
    LocationTagTypeClient locationTagTypeClient;

    @Override
    public Result<PageVo<LocationTagTypeVo>> findLocationTagType(
        LocationTagTypeFindParam locationTagTypeFindParam,
        PageParam pageParam
    ) {
        Specification<LocationTag> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();

            if (StrUtil.isNotEmpty(locationTagTypeFindParam.getCode())) {
                predicates.add(
                    cb.like(root.get(LocationTag.Fields.code), "%" + locationTagTypeFindParam.getCode() + "%")
                );
            }

            if (ObjectUtil.isNotEmpty(locationTagTypeFindParam.getCategory())) {
                predicates.add(cb.equal(root.get(LocationTag.Fields.category), locationTagTypeFindParam.getCategory()));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));

            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };

        List<LocationTag> locationTags = locationTagRepository.findAll(specification);
        List<LocationTagTypeVo> locationTagTypeVos = locationTags
            .stream()
            .map(LocationTagTypeVo::transform)
            .collect(Collectors.toList());
        PageVo<LocationTagTypeVo> pageVo = PageVo.build(locationTagTypeVos, locationTagTypeVos.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<String> sendLocationTagType(LocationTagTypeSendParam locationtagtypeSendParam) {
        ZyLocationTagTypeSendParam zyLocationTagTypeSendParam = ZyLocationTagTypeSendParam
            .builder()
            .orgId(FACTORY_ID)
            .category(Convert.toStr(locationtagtypeSendParam.getCategory()))
            .build();

        String[] split = locationtagtypeSendParam.getIds().split(",");
        ArrayList<Long> ids = ListUtil.toList();
        ArrayList<String> list = ListUtil.toList(split);
        for (String id : list) {
            Optional<LocationTag> locationTagOptional = locationTagRepository.findById(Convert.toLong(id));
            locationTagOptional.ifPresent(locationTag -> ids.add(locationTag.getLinkId()));
        }
        zyLocationTagTypeSendParam.setIds(Convert.toLongArray(ids));

        JSONObject jsonObject = locationTagTypeClient.updateEntityType(
            JSONUtil.toJsonStr(zyLocationTagTypeSendParam),
            ZyUtil.manageLogin()
        );
        if (!ObjectUtil.equals(jsonObject.get("code"), 200)) {
            return Result.no(Status.BAD_REQUEST, StrUtil.toString(jsonObject.get("msg")));
        }

        Specification<LocationTag> specification = (root, cq, cb) -> {
            List<Predicate> predicates = ListUtil.toList();
            if (StrUtil.isNotEmpty(locationtagtypeSendParam.getIds())) {
                CriteriaBuilder.In<Object> in = cb.in(root.get(BasicEntity.Fields.id));
                for (String id : list) in.value(Convert.toLong(id));
                predicates.add(cb.and(in));
            }

            Order order = cb.desc(root.get(BasicEntity.Fields.createTime));
            return cq.orderBy(order).where(predicates.toArray(new Predicate[0])).getRestriction();
        };
        List<LocationTag> locationTags = locationTagRepository.findAll(specification);
        locationTags =
            locationTags
                .stream()
                .peek(locationTag -> locationTag.setCategory(Convert.toInt(locationtagtypeSendParam.getCategory())))
                .collect(Collectors.toList());
        locationTagRepository.saveAll(locationTags);
        return Result.ok();
    }
}
