/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.rehearsal.impl;

import com.yiring.app.domain.rehearsal.AccidentSpot;
import com.yiring.app.domain.rehearsal.AccidentSpotRepository;
import com.yiring.app.param.rehearsal.AccidentSpotAddParam;
import com.yiring.app.param.rehearsal.AccidentSpotConditionParam;
import com.yiring.app.param.rehearsal.AccidentSpotModifyParam;
import com.yiring.app.service.rehearsal.AccidentSpotService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.IdNameVo;
import com.yiring.app.vo.rehearsal.AccidentSpotVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/7 14:02
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class AccidentSpotServiceImpl implements AccidentSpotService {

    @Resource
    private AccidentSpotRepository accidentSpotRepository;

    @Override
    public Result<String> addAccidentSpot(AccidentSpotAddParam param) {
        Boolean exists = accidentSpotRepository.existsByName(param.getName());
        if (exists) {
            return Result.no(Status.EXPECTATION_FAILED, "事故点名称已被占用");
        }
        AccidentSpot accidentSpot = param.transform();
        accidentSpotRepository.saveAndFlush(accidentSpot);
        return Result.ok();
    }

    @Override
    public Result<String> modifyAccidentSpot(AccidentSpotModifyParam param) {
        List<AccidentSpot> allByName = accidentSpotRepository.findAllByName(param.getName());
        if (allByName.size() > 1) {
            log.warn("AccidentSpotServiceImpl.modifyAccidentSpot时查询出两条事故点名称相同的数据,param:[{}]", param);
        }
        AccidentSpot accidentSpot = null;
        for (AccidentSpot item : allByName) {
            if (item.getId().equals(param.getId())) {
                accidentSpot = item;
            } else {
                return Result.no(Status.EXPECTATION_FAILED, "事故点名称已被占用");
            }
        }
        if (accidentSpot == null) {
            Optional<AccidentSpot> optional = accidentSpotRepository.findById(param.getId());
            if (optional.isEmpty()) {
                return Result.no(Status.EXPECTATION_FAILED, "要修改的事故点不存在");
            }
            accidentSpot = optional.get();
        }
        JpaUtil.copyNotNullProperties(param, accidentSpot);
        return Result.ok();
    }

    @Override
    public Result<String> removeAccidentSpot(IdParam param) {
        Optional<AccidentSpot> optional = accidentSpotRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要修改的事故点不存在");
        }
        optional.get().setDeleted(true);
        return Result.ok();
    }

    @Override
    public Result<String> batchRemove(Set<Long> ids) {
        int num = accidentSpotRepository.batchRemove(ids, LocalDateTime.now());
        return Result.ok("删除了" + num + "个事故点");
    }

    @Override
    public Result<AccidentSpotVo> findById(IdParam param) {
        Optional<AccidentSpot> optional = accidentSpotRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要修改的事故点不存在");
        }
        AccidentSpotVo vo = AccidentSpotVo.transform(optional.get());
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<AccidentSpotVo>> findByCondition(
        AccidentSpotConditionParam conditionParam,
        PageParam pageParam
    ) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<AccidentSpot> specification = (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (conditionParam.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get(AccidentSpot.Fields.mapId), conditionParam.getMapId()));
            }
            if (conditionParam.getName() != null) {
                list.add(
                    criteriaBuilder.like(root.get(AccidentSpot.Fields.name), "%" + conditionParam.getName() + "%")
                );
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };

        Page<AccidentSpot> page = accidentSpotRepository.findAll(specification, pageable);
        List<AccidentSpotVo> voList = page.get().map(AccidentSpotVo::transform).collect(Collectors.toList());
        PageVo<AccidentSpotVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<PageVo<IdNameVo>> findAccidentList(String name) {
        List<AccidentSpot> list = accidentSpotRepository.findLikeName(name);
        List<IdNameVo> voList = list
            .stream()
            .map(e -> new IdNameVo(e.getId(), e.getName()))
            .collect(Collectors.toList());
        PageVo<IdNameVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<String> enable(IdParam idParam) {
        Optional<AccidentSpot> optional = accidentSpotRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要启用的事故点不存在");
        }
        optional.get().setEnable(true);
        return Result.ok();
    }

    @Override
    public Result<String> unEnable(IdParam idParam) {
        Optional<AccidentSpot> optional = accidentSpotRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要停用的事故点不存在");
        }
        optional.get().setEnable(false);
        return Result.ok();
    }

    @Override
    public Result<String> batchEnable(Set<Long> ids) {
        int num = accidentSpotRepository.batchEnable(ids, true, LocalDateTime.now());
        return Result.ok("启用了" + num + "个事故点");
    }

    @Override
    public Result<String> batchUnEnable(Set<Long> ids) {
        int num = accidentSpotRepository.batchEnable(ids, false, LocalDateTime.now());
        return Result.ok("停用了" + num + "个事故点");
    }
}
