/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.rehearsal.impl;

import com.yiring.app.domain.rehearsal.EvacuationZone;
import com.yiring.app.domain.rehearsal.EvacuationZoneRepository;
import com.yiring.app.param.rehearsal.EvacuationZoneAddParam;
import com.yiring.app.param.rehearsal.EvacuationZoneConditionParam;
import com.yiring.app.param.rehearsal.EvacuationZoneModifyParam;
import com.yiring.app.service.rehearsal.EvacuationZoneService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.IdNameVo;
import com.yiring.app.vo.rehearsal.EvacuationZoneVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/9 10:42
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class EvacuationZoneServiceImpl implements EvacuationZoneService {

    @Resource
    private EvacuationZoneRepository evacuationZoneRepository;

    @Override
    public Result<String> addOne(EvacuationZoneAddParam param) {
        boolean exists = evacuationZoneRepository.existsByName(param.getName());
        if (exists) {
            return Result.no(Status.EXPECTATION_FAILED, "撤离区名称已存在");
        }
        evacuationZoneRepository.saveAndFlush(param.transform());
        return Result.ok();
    }

    @Override
    public Result<String> modifyOne(EvacuationZoneModifyParam param) {
        List<EvacuationZone> allByName = evacuationZoneRepository.findAllByName(param.getName());
        if (allByName.size() > 1) {
            log.warn("EvacuationZoneServiceImpl.modifyOne查询到多条名称相同的撤离区信息,param:[{}]", param);
        }
        EvacuationZone evacuationZone = null;
        for (EvacuationZone item : allByName) {
            if (item.getId().equals(param.getId())) {
                evacuationZone = item;
            } else {
                return Result.no(Status.EXPECTATION_FAILED, "撤离区名称已存在");
            }
        }
        EvacuationZone transform = param.transform();
        JpaUtil.copyNotNullProperties(transform, evacuationZone);
        return Result.ok();
    }

    @Override
    public Result<String> removeOne(IdParam idParam) {
        Optional<EvacuationZone> optional = evacuationZoneRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要删除的撤离区不存在");
        }
        optional.get().setDeleted(true);
        return Result.ok();
    }

    @Override
    public Result<String> removeMany(Set<Long> ids) {
        int num = evacuationZoneRepository.removeMany(ids, LocalDateTime.now());
        return Result.ok("成功删除了" + num + "条撤离区信息");
    }

    @Override
    public Result<PageVo<IdNameVo>> findEvacuationPullDown(String name) {
        List<EvacuationZone> likeName = evacuationZoneRepository.findLikeName(name);
        List<IdNameVo> voList = likeName
            .stream()
            .map(e -> new IdNameVo(e.getId(), e.getName()))
            .collect(Collectors.toList());
        PageVo<IdNameVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<EvacuationZoneVo> findOne(IdParam idParam) {
        Optional<EvacuationZone> optional = evacuationZoneRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要查询的撤离区不存在");
        }
        EvacuationZoneVo vo = EvacuationZoneVo.transform(optional.get());
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<EvacuationZoneVo>> findList(EvacuationZoneConditionParam param, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<EvacuationZone> specification = (root, query, criteriaBuilder) -> {
            ArrayList<Predicate> list = new ArrayList<>();
            if (param.getMapId() != null) {
                list.add(criteriaBuilder.equal(root.get(EvacuationZone.Fields.mapId), param.getMapId()));
            }
            if (param.getName() != null) {
                list.add(criteriaBuilder.like(root.get(EvacuationZone.Fields.name), "%" + param.getName() + "%"));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
        Page<EvacuationZone> page = evacuationZoneRepository.findAll(specification, pageable);
        List<EvacuationZoneVo> list = page.get().map(EvacuationZoneVo::transform).collect(Collectors.toList());
        PageVo<EvacuationZoneVo> pageVo = PageVo.build(list, list.size());
        return Result.ok(pageVo);
    }

    @Override
    public Result<String> enable(IdParam idParam) {
        Optional<EvacuationZone> optional = evacuationZoneRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要启用的撤离区不存在");
        }
        optional.get().setEnable(true);
        return Result.ok();
    }

    @Override
    public Result<String> unEnable(IdParam idParam) {
        Optional<EvacuationZone> optional = evacuationZoneRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "要停用的撤离区不存在");
        }
        optional.get().setEnable(false);
        return Result.ok();
    }

    @Override
    public Result<String> batchEnable(Set<Long> ids) {
        int num = evacuationZoneRepository.changeEnable(true, ids, LocalDateTime.now());
        return Result.ok("成功启用了" + num + "个撤离区");
    }

    @Override
    public Result<String> batchUnEnable(Set<Long> ids) {
        int num = evacuationZoneRepository.changeEnable(false, ids, LocalDateTime.now());
        return Result.ok("成功停用了" + num + "个撤离区");
    }
}
