/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.video.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.domain.video.Video;
import com.yiring.app.domain.video.VideoRepository;
import com.yiring.app.excel.video.VideoExportExcel;
import com.yiring.app.excel.video.VideoImportExcel;
import com.yiring.app.feign.GbClient;
import com.yiring.app.param.video.StreamProxyItem;
import com.yiring.app.param.video.VideoParam;
import com.yiring.app.param.video.VideoQueryParam;
import com.yiring.app.service.video.VideoService;
import com.yiring.app.util.GeoUtils;
import com.yiring.app.vo.video.VideoNameVo;
import com.yiring.app.vo.video.VideoVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.IndexParam;
import com.yiring.common.vo.PageVo;
import feign.Response;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.locationtech.jts.geom.Point;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author tzl
 * 2022/4/8 15:28
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class VideoServiceImpl implements VideoService {

    @Resource
    VideoRepository videoRepository;

    @Resource
    GbClient gbClient;

    @Resource
    RedisTemplate<String, Object> redisTemplate;

    @Override
    public Result<String> saveVideo(VideoParam videoParam) {
        if (has(videoParam.getUuid())) {
            return Result.no(Status.BAD_REQUEST, "您输入的编号已存在");
        }

        Video video = Video
            .builder()
            .createTime(LocalDateTime.now())
            .m3u8(videoParam.getM3u8())
            .point(GeoUtils.createPoint(videoParam.getLon(), videoParam.getLat()))
            //默认正常
            .videoName(videoParam.getVideoName())
            .status("1")
            .remark(videoParam.getRemark())
            .uuid(videoParam.getUuid())
            .build();
        videoRepository.saveAndFlush(video);
        //还需要同步添加国标28181中对应的摄像头信息
        //标识uuid对应流id,m3u8对应拉流地址,如m3u8中有标注 则将类型切换为FFmpeg,摄像头名称videoName对应名称,流名称默认"stream",超时时间默认60
        StreamProxyItem streamProxyItem = StreamProxyItem
            .builder()
            .name(video.getVideoName())
            .app("stream")
            .stream(video.getUuid())
            .url(video.getM3u8())
            .mediaServerId("ZbnQN5csqxrPix7a")
            .enable(true)
            .enable_hls(true)
            .type(video.getM3u8().contains("sss") ? "ffmpeg" : "default")
            .timeout_ms(video.getM3u8().contains("sss") ? 60 : null)
            .build();
        JSONObject jsonObject = gbClient.proxySave(streamProxyItem, gbCookie());
        if (!jsonObject.get("code").equals(0)) {
            throw new RuntimeException(jsonObject.get("msg").toString());
        }
        return Result.ok();
    }

    @Override
    public Result<String> deleteVideo(IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        //还需要同步删除国标28181中对应的摄像头信息
        JSONObject jsonObject = gbClient.proxyDel("stream", video.getUuid(), gbCookie());
        if (!jsonObject.get("code").equals(0)) {
            throw new RuntimeException(jsonObject.get("msg").toString());
        }
        videoRepository.delete(video);

        return Result.ok();
    }

    @Override
    public Result<String> updateVideo(VideoParam videoParam, IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        if (!video.getUuid().equals(videoParam.getUuid())) {
            //当修改编号时
            if (has(videoParam.getUuid())) {
                return Result.no(Status.BAD_REQUEST, "您输入的编号已存在");
            }
            //当修改跟国标相关联的信息时同步更新国标数据
            //            if (
            //                !video.getVideoName().equals(videoParam.getVideoName()) && !video.getM3u8().equals(videoParam.getM3u8())
            //            ) {
            JSONObject jsonObject = gbClient.proxyDel("stream", video.getUuid(), gbCookie());
            if (!jsonObject.get("code").equals(0)) {
                throw new RuntimeException(jsonObject.get("msg").toString());
            }
        }
        //        }
        BeanUtils.copyProperties(videoParam, video);
        video.setPoint(GeoUtils.createPoint(videoParam.getLon(), videoParam.getLat()));
        videoRepository.saveAndFlush(video);
        //将信息发送到队列,等待同步至国标
        //        new Thread(() -> {
        StreamProxyItem streamProxyItem = StreamProxyItem
            .builder()
            .name(video.getVideoName())
            .app("stream")
            .stream(video.getUuid())
            .url(video.getM3u8())
            .mediaServerId("ZbnQN5csqxrPix7a")
            .enable(true)
            .enable_hls(true)
            .type(video.getM3u8().contains("sss") ? "ffmpeg" : "default")
            .timeout_ms(video.getM3u8().contains("sss") ? 60 : null)
            .build();
        JSONObject jsonObject = gbClient.proxySave(streamProxyItem, gbCookie());
        if (!jsonObject.get("code").equals(0)) {
            throw new RuntimeException(jsonObject.get("msg").toString());
        }
        //        })
        //            .start();
        return Result.ok();
    }

    @Override
    public Result<Point> selectPoint(IdParam idParam) {
        Optional<Video> optional = videoRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Video video = optional.get();
        return Result.ok(video.getPoint());
    }

    @Override
    public Result<PageVo<VideoVo>> pageVideo(VideoQueryParam videoQueryParam) {
        PageVo<VideoVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Video.Fields.createTime));
        //如果传分页参数则分页,否查全部数据
        if (Objects.nonNull(videoQueryParam.getPageNo()) && Objects.nonNull(videoQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(videoQueryParam.getPageNo() - 1, videoQueryParam.getPageSize());
            Page<Video> all = videoRepository.findAll(condition(videoQueryParam), pageable);
            List<VideoVo> data = all
                .get()
                .map(car -> {
                    VideoVo vo = new VideoVo();
                    BeanUtils.copyProperties(car, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Video> all = videoRepository.findAll(condition(videoQueryParam), sort);
            List<VideoVo> data = all
                .stream()
                .map(video -> {
                    VideoVo vo = new VideoVo();
                    BeanUtils.copyProperties(video, vo);
                    //                    vo.setPoint(video.getPoint());
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    /**
     * 条件查询
     *
     * @param videoQueryParam 筛选条件
     * @return Specification
     */
    @Override
    public Specification<Video> condition(VideoQueryParam videoQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (StrUtil.isNotBlank(videoQueryParam.getVideoName())) {
                list.add(
                    criteriaBuilder.like(
                        root.get("videoName").as(String.class),
                        "%" + videoQueryParam.getVideoName() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(videoQueryParam.getUuid())) {
                list.add(
                    criteriaBuilder.like(root.get("uuid").as(String.class), "%" + videoQueryParam.getUuid() + "%")
                );
            }

            if (StrUtil.isNotBlank(videoQueryParam.getStatus())) {
                list.add(criteriaBuilder.equal(root.get("uuid").as(String.class), videoQueryParam.getStatus()));
            }
            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }

    @Override
    public void exportVideo(VideoQueryParam videoQueryParam, HttpServletResponse response) {
        Specification<Video> specification = condition(videoQueryParam);
        List<Video> all = videoRepository.findAll(specification);
        List<VideoExportExcel> data = all
            .stream()
            .map(video -> {
                VideoExportExcel videoExportExcel = new VideoExportExcel();
                BeanUtils.copyProperties(video, videoExportExcel);
                videoExportExcel.setLon(video.getPoint().getX());
                videoExportExcel.setLat(video.getPoint().getY());
                return videoExportExcel;
            })
            .collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<VideoExportExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(VideoExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(data);

            String fileName = URLEncoder.encode("监控点位来访信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出监控点位来访信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> importVideo(MultipartFile file) {
        if (file.isEmpty()) {
            return Result.no(Status.BAD_REQUEST, "请选择文件");
        }
        List<VideoImportExcel> videoImportExcels;
        try (InputStream inputStream = file.getInputStream()) {
            videoImportExcels =
                SaxExcelReader
                    .of(VideoImportExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            return Result.no(Status.BAD_REQUEST, "文件导入异常");
        }

        List<Video> videos = videoImportExcels
            .stream()
            .map(videoImportExcel -> {
                Video video = new Video();
                BeanUtils.copyProperties(videoImportExcel, video);
                video.setPoint(GeoUtils.createPoint(videoImportExcel.getLon(), videoImportExcel.getLat()));
                video.setCreateTime(LocalDateTime.now());
                return video;
            })
            .toList();

        if (videos != null) {
            videos.forEach(video -> {
                videoRepository
                    .findOne((root, cq, cb) -> {
                        List<Predicate> predicates = new ArrayList<>();
                        if (StrUtil.isNotBlank(video.getUuid())) {
                            predicates.add(cb.equal(root.get("uuid").as(String.class), video.getUuid()));
                        }
                        return cq.where(predicates.toArray(new Predicate[0])).getRestriction();
                    })
                    .ifPresent(one -> {
                        //如果存在则操作修改
                        video.setId(one.getId());
                        //删除国标关联监控信息
                        gbClient.proxyDel("stream", video.getUuid(), gbCookie());
                    });
                videoRepository.saveAndFlush(video);
                //导入监控点位后,还需要同步数据到国标28181
                StreamProxyItem streamProxyItem = StreamProxyItem
                    .builder()
                    .name(video.getVideoName())
                    .app("stream")
                    .stream(video.getUuid())
                    .url(video.getM3u8())
                    .mediaServerId("ZbnQN5csqxrPix7a")
                    .enable(true)
                    .enable_hls(true)
                    .type(video.getM3u8().contains("sss") ? "default" : "ffmpeg")
                    .timeout_ms(video.getM3u8().contains("sss") ? 60 : null)
                    .build();
                JSONObject jsonObject = gbClient.proxySave(streamProxyItem, gbCookie());
                if (!jsonObject.get("code").equals(0)) {
                    throw new RuntimeException(jsonObject.get("msg").toString());
                }
            });
        }
        return Result.ok();
    }

    @Override
    public Result<VideoNameVo> findByVideoNameLike(IndexParam indexParam) {
        VideoNameVo videoNameVo = new VideoNameVo();
        Video byVideoNameLike = videoRepository.findByVideoNameLike(indexParam.getStr());
        BeanUtils.copyProperties(byVideoNameLike, videoNameVo);
        return Result.ok(videoNameVo);
    }

    boolean has(String uuid) {
        Video video = Video.builder().uuid(uuid).build();
        return videoRepository.count(Example.of(video)) > 0;
    }

    /**
     * 获取国标系统登录cookie
     *
     * @return cookie
     */
    String gbCookie() {
        Object gbCookie = redisTemplate.opsForValue().get("gb_cookie");
        if (ObjectUtil.isNotEmpty(gbCookie)) {
            return String.valueOf(gbCookie);
        }
        Response response = gbClient.userLogin("admin", "21232f297a57a5a743894a0e4a801fc3");
        Collection<String> strings = response.headers().get("Set-Cookie");
        String jsessionid = strings.stream().filter(a -> a.contains("JSESSIONID")).collect(Collectors.joining(", "));
        redisTemplate.opsForValue().set("gb_cookie", jsessionid, 30, TimeUnit.MINUTES);
        return jsessionid;
    }
}
